-- Start environemt setup : To be executed in SQL Server 2000 and 2005
-- Create a test database
CREATE DATABASE MyTestDatabase
GO

USE MyTestDatabase
GO

-- Create Departments table
CREATE TABLE Departments
(
	DeptId int IDENTITY(1,1) NOT NULL PRIMARY KEY ,
	DeptName varchar(50)
)
GO

-- Create Employees table
CREATE TABLE Employees
(
	EmpId int IDENTITY(1,1) NOT NULL PRIMARY KEY ,
	EmpName varchar(50) ,
	DeptId int CONSTRAINT FOREIGN_KEY REFERENCES Departments (DeptId) ,
	EmpSalary money
)
GO

SET NOCOUNT ON

-- Populate Departments table with test data
INSERT INTO Departments (DeptName)
VALUES ('HR')

INSERT INTO Departments (DeptName)
VALUES ('Payroll')

INSERT INTO Departments (DeptName)
VALUES ('Admin')

-- Populate Employees table with test data
INSERT INTO Employees (EmpName, DeptId, EmpSalary)
VALUES ('John', 1, 5000)

INSERT INTO Employees (EmpName, DeptId, EmpSalary)
VALUES ('Albert', 1, 4500)

INSERT INTO Employees (EmpName, DeptId, EmpSalary)
VALUES ('Crain', 2, 6000)

INSERT INTO Employees (EmpName, DeptId, EmpSalary)
VALUES ('Micheal', 2, 5000)

SET NOCOUNT OFF

GO

-- Create udf fn_DepartmentEmployees that accepts department id and 
-- returns the list of employee's name and salary along with the department id.
CREATE FUNCTION fn_DepartmentEmployees (@DeptId int)
RETURNS @DeptEmp TABLE (DeptId int, EmpName varchar(50), EmpSalary money)
BEGIN
	INSERT INTO @DeptEmp (DeptId, EmpName, EmpSalary)
	SELECT DeptId, EmpName, EmpSalary
	FROM Employees
	WHERE DeptId = @DeptId

	RETURN
END
GO

-- End  environemt setup

/*
-- Execute listing 1, 2 and 3 in SQL Server 2000 for understanding table-valued function limitation 

-- Listing 1
SELECT DeptId, EmpName, EmpSalary 
FROM fn_DepartmentEmployees (1)
GO

-- Listing 2
SELECT DeptName, EmpName, EmpSalary
FROM fn_DepartmentEmployees (1) de
JOIN Departments d ON d.DeptId = de.DeptId
GO

-- Listing 3
SELECT DeptName, EmpName, EmpSalary
FROM Departments d 
JOIN fn_DepartmentEmployees (d.DeptId) de ON d.DeptId = de.DeptId
GO
*/


/*
-- Execute listings 4 & 5 in SQL Server 2005 to understand the new APPLY construct 
-- that overcomes the SQL Server 2000 limitation of table-valued function

-- Listing 4
-- CROSS APPLY query
SELECT DeptName, EmpName, EmpSalary
FROM Departments d 
CROSS APPLY fn_DepartmentEmployees (d.DeptId)
GO

-- Listing 5
-- OUTER APPLY query
SELECT DeptName, EmpName, EmpSalary
FROM Departments d 
OUTER APPLY fn_DepartmentEmployees (d.DeptId)
GO
*/
